/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React, { useState } from 'react';

import {
  EuiButton,
  EuiButtonIcon,
  EuiFlexGroup,
  EuiFlexItem,
  EuiIcon,
  EuiLink,
  EuiPanel,
  EuiPopover,
  EuiPopoverFooter,
  EuiPopoverTitle,
  EuiText,
  EuiTextColor,
  EuiTitle,
  useEuiTheme,
} from '@elastic/eui';

import { i18n } from '@kbn/i18n';
import { css } from '@emotion/react';
import { MetaDataProps } from './result_types';

interface Props {
  metaData: MetaDataProps;
  title: string;
  rightSideActions?: React.ReactNode;
  showScore?: boolean;
  onTitleClick?: () => void;
}

interface TermDef {
  label: string | number;
}

const Term: React.FC<TermDef> = ({ label }) => (
  <EuiFlexItem grow={false}>
    <strong>
      <EuiTextColor color="subdued">{label}:</EuiTextColor>
    </strong>
  </EuiFlexItem>
);

const Definition: React.FC<TermDef> = ({ label }) => (
  <EuiFlexItem>
    <EuiTextColor color="subdued">{label}</EuiTextColor>
  </EuiFlexItem>
);
const MetadataPopover: React.FC<MetaDataProps> = ({
  id,
  onDocumentDelete,
  score,
  showScore = false,
}) => {
  const [popoverIsOpen, setPopoverIsOpen] = useState(false);
  const closePopover = () => setPopoverIsOpen(false);

  const metaDataIcon = (
    <EuiButtonIcon
      display="empty"
      size="s"
      iconType="iInCircle"
      color="primary"
      data-test-subj="documentMetadataButton"
      onClick={(e: React.MouseEvent<HTMLElement>) => {
        e.stopPropagation();
        setPopoverIsOpen(!popoverIsOpen);
      }}
      aria-label={i18n.translate('searchIndexDocuments.result.header.metadata.icon.ariaLabel', {
        defaultMessage: 'Metadata for document: {id}',
        values: { id },
      })}
    />
  );

  return (
    <EuiPopover button={metaDataIcon} isOpen={popoverIsOpen} closePopover={closePopover}>
      <EuiPopoverTitle>
        {i18n.translate('searchIndexDocuments.result.header.metadata.title', {
          defaultMessage: 'Document metadata',
        })}
      </EuiPopoverTitle>
      <EuiFlexGroup
        gutterSize="s"
        direction="column"
        css={css`
          width: 20rem;
        `}
      >
        <EuiFlexItem>
          <EuiFlexGroup justifyContent="spaceBetween" gutterSize="s">
            <Term label="ID" />
            <Definition label={id} />
          </EuiFlexGroup>
        </EuiFlexItem>

        {score && showScore && (
          <EuiFlexItem>
            <EuiFlexGroup justifyContent="spaceBetween" gutterSize="s">
              <Term
                label={i18n.translate('searchIndexDocuments.result.header.metadata.score', {
                  defaultMessage: 'Score',
                })}
              />
              <Definition label={score?.toString()} />
            </EuiFlexGroup>
          </EuiFlexItem>
        )}
      </EuiFlexGroup>
      {onDocumentDelete && (
        <EuiPopoverFooter>
          <EuiButton
            iconType="trash"
            color="danger"
            size="s"
            data-test-subj="deleteDocumentButton"
            onClick={(e: React.MouseEvent<HTMLElement>) => {
              e.stopPropagation();
              onDocumentDelete();
              closePopover();
            }}
            fullWidth
          >
            {i18n.translate('searchIndexDocuments.result.header.metadata.deleteDocument', {
              defaultMessage: 'Delete document',
            })}
          </EuiButton>
        </EuiPopoverFooter>
      )}
    </EuiPopover>
  );
};

const Score: React.FC<{ score: MetaDataProps['score'] }> = ({ score }) => {
  return (
    <EuiPanel paddingSize="xs" hasShadow={false} color="subdued" grow>
      <EuiFlexGroup
        direction="column"
        responsive={false}
        alignItems="center"
        justifyContent="center"
        gutterSize="s"
      >
        <EuiFlexItem grow>
          <EuiIcon type="visGauge" size="m" />
        </EuiFlexItem>
        <EuiFlexItem grow>
          <EuiPanel
            hasShadow={false}
            hasBorder={false}
            css={css`
              inline-size: 5ch;
              max-inline-size: 100%;
            `}
            paddingSize="none"
            color="subdued"
          >
            <EuiText textAlign="center" size="xs">
              {score ? score.toString().substring(0, 5) : '-'}
            </EuiText>
          </EuiPanel>
        </EuiFlexItem>
      </EuiFlexGroup>
    </EuiPanel>
  );
};

export const RichResultHeader: React.FC<Props> = ({
  title,
  metaData,
  rightSideActions = null,
  showScore = false,
  onTitleClick,
}) => {
  const { euiTheme } = useEuiTheme();
  return (
    <EuiFlexItem
      grow
      css={css`
        min-height: ${euiTheme.base * 1}px;
        max-height: ${euiTheme.base * 8}px;
      `}
    >
      <EuiFlexGroup gutterSize="m" alignItems="center">
        {showScore && (
          <EuiFlexItem grow={false}>
            <Score score={metaData.score} />
          </EuiFlexItem>
        )}
        <EuiFlexItem>
          <EuiFlexGroup alignItems="center">
            <EuiFlexItem>
              <EuiText>
                <EuiFlexGroup alignItems="center" gutterSize="l" responsive={false}>
                  <EuiFlexItem>
                    {onTitleClick ? (
                      <EuiLink onClick={onTitleClick} color="text">
                        <EuiTitle size="s">
                          <h4>{title}</h4>
                        </EuiTitle>
                      </EuiLink>
                    ) : (
                      <EuiTitle size="s">
                        <h4>{title}</h4>
                      </EuiTitle>
                    )}
                  </EuiFlexItem>
                  {!!metaData && (
                    <EuiFlexItem grow={false}>
                      <MetadataPopover {...metaData} showScore={showScore} />
                    </EuiFlexItem>
                  )}
                </EuiFlexGroup>
              </EuiText>
            </EuiFlexItem>
          </EuiFlexGroup>
        </EuiFlexItem>
        <EuiFlexItem grow={false}>{rightSideActions}</EuiFlexItem>
      </EuiFlexGroup>
    </EuiFlexItem>
  );
};
