/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

jest.mock('./nav', () => ({
  useAppSearchNav: () => [],
}));
import '../../../__mocks__/shallow_useeffect.mock';
import { setMockValues } from '../../../__mocks__/kea_logic';

import React from 'react';

import { shallow, mount } from 'enzyme';
import { of } from 'rxjs';

import { __IntlProvider as IntlProvider } from '@kbn/i18n-react';

import { SetAppSearchChrome } from '../../../shared/kibana_chrome';
import { EnterpriseSearchPageTemplateWrapper } from '../../../shared/layout';
import { SendAppSearchTelemetry } from '../../../shared/telemetry';

import { AppSearchPageTemplate } from './page_template';

const mockValues = {
  getChromeStyle$: () => of('classic'),
  updateSideNavDefinition: jest.fn(),
};

describe('AppSearchPageTemplate', () => {
  beforeEach(() => {
    setMockValues({ ...mockValues });
  });

  it('renders', () => {
    const wrapper = shallow(
      <AppSearchPageTemplate>
        <div className="hello">world</div>
      </AppSearchPageTemplate>
    );

    expect(wrapper.type()).toEqual(EnterpriseSearchPageTemplateWrapper);
    expect(wrapper.prop('solutionNav')).toEqual({ name: 'App Search', items: [] });
    expect(wrapper.find('.hello').text()).toEqual('world');
  });

  describe('page chrome', () => {
    it('takes a breadcrumb array & renders a product-specific page chrome', () => {
      const wrapper = shallow(<AppSearchPageTemplate pageChrome={['Some page']} />);
      const setPageChrome = wrapper
        .find(EnterpriseSearchPageTemplateWrapper)
        .prop('setPageChrome') as any;

      expect(setPageChrome.type).toEqual(SetAppSearchChrome);
      expect(setPageChrome.props.trail).toEqual(['Some page']);
    });
  });

  describe('page telemetry', () => {
    it('takes a metric & renders product-specific telemetry viewed event', () => {
      const wrapper = shallow(<AppSearchPageTemplate pageViewTelemetry="some_page" />);

      expect(wrapper.find(SendAppSearchTelemetry).prop('action')).toEqual('viewed');
      expect(wrapper.find(SendAppSearchTelemetry).prop('metric')).toEqual('some_page');
    });
  });

  it('passes down any ...pageTemplateProps that EnterpriseSearchPageTemplateWrapper accepts', () => {
    const wrapper = shallow(
      <AppSearchPageTemplate
        pageHeader={{ pageTitle: 'hello world' }}
        isLoading={false}
        emptyState={<div />}
      />
    );

    expect(wrapper.find(EnterpriseSearchPageTemplateWrapper).prop('pageHeader')!.pageTitle).toEqual(
      'hello world'
    );
    expect(wrapper.find(EnterpriseSearchPageTemplateWrapper).prop('isLoading')).toEqual(false);
    expect(wrapper.find(EnterpriseSearchPageTemplateWrapper).prop('emptyState')).toEqual(<div />);
  });

  describe('deprecation callout', () => {
    it('renders the deprecation callout', () => {
      const wrapper = shallow(<AppSearchPageTemplate />);
      expect(wrapper.find('EnterpriseSearchDeprecationCallout')).toHaveLength(1);
    });

    it('dismisses the deprecation callout', () => {
      const wrapper = mount(
        <IntlProvider locale="en">
          <AppSearchPageTemplate />
        </IntlProvider>
      );

      sessionStorage.setItem('appSearchHideDeprecationCallout', 'false');
      expect(wrapper.find('EnterpriseSearchDeprecationCallout')).toHaveLength(1);

      wrapper.find('button[data-test-subj="euiDismissCalloutButton"]').simulate('click');
      expect(wrapper.find('EnterpriseSearchDeprecationCallout')).toHaveLength(0);
      expect(sessionStorage.getItem('appSearchHideDeprecationCallout')).toEqual('true');
    });

    it('does not render the deprecation callout if dismissed', () => {
      const wrapper = shallow(<AppSearchPageTemplate />);
      sessionStorage.setItem('appSearchHideDeprecationCallout', 'true');
      expect(wrapper.find('EnterpriseSearchDeprecationCallout')).toHaveLength(0);
    });
  });
});
