/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React from 'react';

import { useValues } from 'kea';

import { EuiFlexGroup, EuiFlexItem } from '@elastic/eui';
import { i18n } from '@kbn/i18n';

import { INGESTION_METHOD_IDS } from '../../../../../common/constants';

import { ProductFeatures } from '../../../../../common/types';

import { HttpLogic } from '../../../shared/http';
import { KibanaLogic } from '../../../shared/kibana/kibana_logic';

import { NEW_API_PATH, NEW_CRAWLER_PATH, NEW_INDEX_SELECT_CONNECTOR_PATH } from '../../routes';
import { EnterpriseSearchContentPageTemplate } from '../layout/page_template';
import { CannotConnect } from '../search_index/components/cannot_connect';
import { baseBreadcrumbs } from '../search_indices';

import { NewIndexCard } from './new_index_card';

const getAvailableMethodOptions = (productFeatures: ProductFeatures): INGESTION_METHOD_IDS[] => {
  return [
    INGESTION_METHOD_IDS.API,
    ...(productFeatures.hasWebCrawler ? [INGESTION_METHOD_IDS.CRAWLER] : []),
    ...(productFeatures.hasConnectors ? [INGESTION_METHOD_IDS.CONNECTOR] : []),
  ];
};

export const NewIndex: React.FC = () => {
  const { config, productFeatures } = useValues(KibanaLogic);
  const availableIngestionMethodOptions = getAvailableMethodOptions(productFeatures);
  const { errorConnectingMessage } = useValues(HttpLogic);

  return (
    <EnterpriseSearchContentPageTemplate
      pageChrome={[
        ...baseBreadcrumbs,
        i18n.translate('xpack.enterpriseSearch.content.newIndex.breadcrumb', {
          defaultMessage: 'New ingestion method',
        }),
      ]}
      pageViewTelemetry="New Index"
      isLoading={false}
      pageHeader={{
        description: i18n.translate('xpack.enterpriseSearch.content.newIndex.pageDescription', {
          defaultMessage:
            'Create a search optimized Elasticsearch index to store your content. Start by selecting an ingestion method.',
        }),
        pageTitle: i18n.translate('xpack.enterpriseSearch.content.newIndex.pageTitle', {
          defaultMessage: 'Select an ingestion method',
        }),
      }}
    >
      <EuiFlexGroup direction="column">
        {errorConnectingMessage && productFeatures.hasWebCrawler && <CannotConnect />}
        <>
          <EuiFlexItem>
            <EuiFlexGroup>
              {availableIngestionMethodOptions.map((type) => (
                <EuiFlexItem key={type}>
                  <NewIndexCard
                    disabled={Boolean(
                      type === INGESTION_METHOD_IDS.CRAWLER &&
                        (errorConnectingMessage || !config.host)
                    )}
                    type={type}
                    onSelect={() => {
                      if (type === INGESTION_METHOD_IDS.CONNECTOR) {
                        KibanaLogic.values.navigateToUrl(NEW_INDEX_SELECT_CONNECTOR_PATH);
                      } else if (type === INGESTION_METHOD_IDS.CRAWLER) {
                        KibanaLogic.values.navigateToUrl(NEW_CRAWLER_PATH);
                      } else {
                        KibanaLogic.values.navigateToUrl(NEW_API_PATH);
                      }
                    }}
                  />
                </EuiFlexItem>
              ))}
            </EuiFlexGroup>
          </EuiFlexItem>
        </>
      </EuiFlexGroup>
    </EnterpriseSearchContentPageTemplate>
  );
};
